import pytest

from astdoc.parser import Parser
from astdoc.renderer import TemplateKind


@pytest.fixture(autouse=True)
def _load_templates():
    from astdoc.renderer import load_templates

    load_templates()


def test_load_templates():
    from astdoc.renderer import templates

    assert "heading" in templates
    assert "object" in templates
    assert "document" in templates
    assert "source" in templates


def test_render_heading_module():
    from astdoc.renderer import render_heading

    parser = Parser.create("examples._styles.google")
    assert parser
    name_set = parser.parse_name_set()
    m = render_heading(name_set, 1)
    assert '<h1 class="astdoc-heading" id="examples._styles.google" markdown="1">' in m
    assert ">examples.\\_styles.google</h1>" in m


def test_render_heading_export():
    from astdoc.renderer import render_heading

    parser = Parser.create("jinja2.Template")
    assert parser
    name_set = parser.parse_name_set()
    m = render_heading(name_set, 1)
    assert '<h1 class="astdoc-heading" id="jinja2.Template" markdown="1">' in m


def test_render_heading_alias():
    from astdoc.renderer import render_heading

    parser = Parser.create("examples._styles.ExampleClassGoogle")
    assert parser
    name_set = parser.parse_name_set()
    m = render_heading(name_set, 1)
    assert 'id="examples._styles.ExampleClassGoogle"' in m
    assert ">examples.\\_styles.ExampleClassGoogle</h1>" in m


def test_render_object_module():
    from astdoc.renderer import render_object

    parser = Parser.create("examples._styles")
    assert parser
    name_set = parser.parse_name_set()
    m = render_object(name_set, 1, "object", [])
    x = '<p class="astdoc-object astdoc-page-object" id="examples._styles" markdown="1">'
    assert x in m
    x = "[object][__astdoc__.__object__.examples._styles]"
    assert x in m


def test_render_object_object():
    from astdoc.renderer import render_object

    parser = Parser.create("astdoc.node.Node")
    assert parser
    name_set = parser.parse_name_set()
    m = render_object(name_set, 1, "source", [])
    x = "[source][__astdoc__.__source__.astdoc"
    assert x in m
    assert "[Node][__astdoc__.astdoc.node.Node]" in m


def test_render_source_module():
    from astdoc.renderer import render_source

    parser = Parser.create("astdoc.ast")
    assert parser
    m = render_source(parser.obj)
    assert m.startswith('``` {.python .astdoc-source .no-copy linenums="1"}\n')
    assert "Iterator[AST]:## __astdoc__.astdoc.ast.iter_child_nodes" in m
    assert "def _iter_assign_nodes(## __astdoc__.astdoc.ast._iter_assign_nodes" in m


def test_render_source_invalid():
    from astdoc.renderer import render_source

    assert render_source(None) == ""  # type: ignore


@pytest.mark.parametrize("level", [1, 2])
@pytest.mark.parametrize("source", [True, False])
def test_render_module(level: int, source: bool):
    from astdoc.renderer import render

    def predicate(parser: Parser, kind: TemplateKind):
        if source:
            return True

        return kind != TemplateKind.SOURCE

    m = render("astdoc.ast", None, level, "object", predicate)
    assert f'<h{level} class="astdoc-heading" id="astdoc.ast" markdown="1">' in m

    if source:
        assert "```" in m
    else:
        assert "```" not in m

    assert '<div class="astdoc-document" markdown="1">' in m


def test_render_module_invalid():
    from astdoc.renderer import render

    m = render("astdoc.invalid", None, 1, "object", None)
    assert "!!! failure" in m
    assert "'astdoc.invalid' not found." in m


def test_get_source_id_must_be_unique():
    from astdoc.node import iter_module_members
    from astdoc.object import Module, get_object
    from astdoc.renderer import _get_source

    obj = get_object("astdoc.object")
    assert isinstance(obj, Module)
    s = _get_source(obj)

    for name, _ in iter_module_members("astdoc.object"):
        assert s.count(f"## __astdoc__.astdoc.object.{name}\n") == 1
